'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handler = exports.builder = exports.describe = exports.command = undefined;

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _fs = require('fs');

var _apiClient = require('../apiClient');

var _formatError = require('../formatError');

var _gatherFiles = require('../gatherFiles');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var command = exports.command = 'upload <paths..>';
var describe = exports.describe = 'Upload JavaScript sourcemaps for a release';
var builder = exports.builder = function builder(args) {
  args.usage('\nUsage: logrocket [-k <apikey>] upload [-r <release>] <paths..>').option('r', {
    alias: 'release',
    type: 'string',
    describe: 'The release version for these files',
    demand: 'You must specify a release, use -r or --release'
  }).option('p', {
    alias: 'urlPrefix',
    type: 'string',
    default: '~/',
    describe: 'Sets a URL prefix in front of all files. Defaults to "~/"'
  }).demand(1, 'Missing upload path: e.g. logrocket upload -r 1.2.3 dist/').option('gcs-token', { // for testing, pass the webhook token to get an immediate pending=no
    type: 'string',
    describe: false
  }).option('gcs-bucket', { // for testing, pass the webhook bucket to get an immediate pending=no
    type: 'string',
    describe: false
  }).implies({
    'gcs-token': 'gcs-bucket',
    'gcs-bucket': 'gcs-token'
  }).option('max-retries', {
    type: 'number',
    describe: 'Failed upload retry limit (0 disables)',
    default: 0
  }).option('max-retry-delay', {
    type: 'number',
    describe: 'Maximum delay between retries in ms',
    default: 30000
  }).help('help');
};

var handler = exports.handler = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(args) {
    var paths, release, apikey, apihost, verbose, urlPrefix, client, uploadFile, fileList, CHUNK_SIZE, i;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            paths = args.paths, release = args.release, apikey = args.apikey, apihost = args.apihost, verbose = args.verbose, urlPrefix = args.urlPrefix;


            console.info('Preparing to upload sourcemaps for release ' + release + ' ...');
            console.info('Gathering file list...');

            client = (0, _apiClient.apiClient)({ apikey: apikey, apihost: apihost });
            _context2.next = 6;
            return client.checkStatus();

          case 6:

            if (args['gcs-token']) {
              client.setGCSData({
                gcsToken: args['gcs-token'],
                gcsBucket: args['gcs-bucket']
              });
            }

            uploadFile = function () {
              var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(_ref3) {
                var path = _ref3.path,
                    name = _ref3.name;
                var filepath, data, res;
                return _regenerator2.default.wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        console.info('Uploading: ' + name);

                        filepath = urlPrefix.replace(/\/$/, '') + '/' + name;
                        data = {
                          contents: (0, _fs.createReadStream)(path),
                          data: { filepath: filepath },
                          maxRetries: args['max-retries'],
                          maxRetryDelay: args['max-retry-delay'],
                          url: 'releases/' + release + '/artifacts'
                        };
                        _context.prev = 3;
                        _context.next = 6;
                        return client.uploadFile(data);

                      case 6:
                        res = _context.sent;

                        if (res.ok) {
                          _context.next = 11;
                          break;
                        }

                        console.error('Failed to upload: ' + name);
                        _context.next = 11;
                        return (0, _formatError.formatError)(res, { verbose: verbose });

                      case 11:
                        _context.next = 17;
                        break;

                      case 13:
                        _context.prev = 13;
                        _context.t0 = _context['catch'](3);

                        console.error(_context.t0.message);
                        process.exit(1);

                      case 17:
                      case 'end':
                        return _context.stop();
                    }
                  }
                }, _callee, undefined, [[3, 13]]);
              }));

              return function uploadFile(_x2) {
                return _ref2.apply(this, arguments);
              };
            }();

            _context2.next = 10;
            return (0, _gatherFiles.gatherFiles)(paths);

          case 10:
            fileList = _context2.sent;


            console.info('Found ' + fileList.length + ' file' + (fileList.length === 1 ? '' : 's') + ' ...');
            if (verbose) {
              console.info(fileList.map(function (_ref4) {
                var path = _ref4.path;
                return '- ' + path;
              }).join('\n'));
            } else {
              console.info('Rerun command with --verbose to see file paths');
            }

            CHUNK_SIZE = 10;
            i = 0;

          case 15:
            if (!(i < fileList.length)) {
              _context2.next = 21;
              break;
            }

            _context2.next = 18;
            return _promise2.default.all(fileList.slice(i, i + CHUNK_SIZE).map(uploadFile));

          case 18:
            i += CHUNK_SIZE;
            _context2.next = 15;
            break;

          case 21:

            console.info('Success!');

          case 22:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  }));

  return function handler(_x) {
    return _ref.apply(this, arguments);
  };
}();