'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getMachOArchs = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _values = require('babel-runtime/core-js/object/values');

var _values2 = _interopRequireDefault(_values);

var _ARCH_BASE_NAMES, _ARCH_SUB_NAMES;

var getArchEntry = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(fd, magic) {
    var archOffset = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
    var archNum = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : null;
    var archName = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : null;

    var isMach64Header, headerSize, shouldSwap, archSuffix, header, _getHeaderVals, cpuType, cpuSubType, ncmds, arch, offset, cmdNum, cmdSuffix, loadCommand, _getLoadCommandDetail, cmd, cmdSize, uuidBuffer;

    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            isMach64Header = getIsMach64Header(magic);
            headerSize = isMach64Header ? MACH_64_HEADER_BYTES : MACH_HEADER_BYTES;
            shouldSwap = shouldSwapBytes(magic);
            archSuffix = getErrorSuffixForArch(archNum, archName);
            _context.next = 6;
            return readBytes(fd, archOffset, headerSize, 'Error reading header' + archSuffix);

          case 6:
            header = _context.sent;
            _getHeaderVals = getHeaderVals(header, shouldSwap, archSuffix), cpuType = _getHeaderVals.cpuType, cpuSubType = _getHeaderVals.cpuSubType, ncmds = _getHeaderVals.ncmds;
            arch = archName || getArchName(cpuType, cpuSubType);


            archSuffix = getErrorSuffixForArch(archNum, arch);

            offset = archOffset + headerSize;
            cmdNum = 0;

          case 12:
            if (!(cmdNum < ncmds)) {
              _context.next = 29;
              break;
            }

            cmdSuffix = 'load command ' + cmdNum + ' ' + archSuffix;
            _context.next = 16;
            return readBytes(fd, offset, LOAD_COMMAND_BYTES, 'Error reading ' + cmdSuffix);

          case 16:
            loadCommand = _context.sent;
            _getLoadCommandDetail = getLoadCommandDetails(loadCommand, shouldSwap, cmdSuffix), cmd = _getLoadCommandDetail.cmd, cmdSize = _getLoadCommandDetail.cmdSize;

            if (!(cmdSize < LOAD_COMMAND_BYTES)) {
              _context.next = 20;
              break;
            }

            throw new _errorTypes.LoadCommandError('Parsed total command size ' + cmdSize + ' smaller than minimum ' + LOAD_COMMAND_BYTES);

          case 20:
            if (!(cmd === LC_UUID)) {
              _context.next = 25;
              break;
            }

            _context.next = 23;
            return readBytes(fd, offset + LOAD_COMMAND_BYTES, UUID_BYTES, 'Error reading uuid of ' + cmdSuffix, false);

          case 23:
            uuidBuffer = _context.sent;
            return _context.abrupt('return', {
              uuid: uuidBuffer.toString('hex'),
              arch: arch,
              fileFormat: 'macho'
            });

          case 25:

            offset += cmdSize;

          case 26:
            cmdNum++;
            _context.next = 12;
            break;

          case 29:
            throw new _errorTypes.MissingUUIDError('No mapping uuid found ' + archSuffix + ' at offset ' + archOffset);

          case 30:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function getArchEntry(_x6, _x7) {
    return _ref.apply(this, arguments);
  };
}();

var getMachOArchs = exports.getMachOArchs = function () {
  var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(filepath) {
    var _this = this;

    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            return _context3.abrupt('return', new _promise2.default(function (resolve, reject) {
              (0, _fs.open)(filepath, 'r', function () {
                var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(err, fd) {
                  var magicDataView, magic, archEntries, header, numArchs, offset, archNum, fatArchHeader, _getArchDetails, cpuType, cpuSubType, archOffset, archName, archEntry, _archEntry;

                  return _regenerator2.default.wrap(function _callee2$(_context2) {
                    while (1) {
                      switch (_context2.prev = _context2.next) {
                        case 0:
                          _context2.prev = 0;

                          if (err) {
                            (0, _errorTypes.handleFileError)('Error opening file ' + filepath, err);
                          }

                          _context2.next = 4;
                          return readBytes(fd, 0, MAGIC_NUMBER_BYTES, 'Error getting magic number');

                        case 4:
                          magicDataView = _context2.sent;
                          magic = parseMagicNumber(magicDataView);

                          if (!getIsMultiArch(magic)) {
                            _context2.next = 35;
                            break;
                          }

                          archEntries = [];
                          _context2.next = 10;
                          return readBytes(fd, 0, FAT_HEADER_BYTES, 'Error reading multi-arch header');

                        case 10:
                          header = _context2.sent;
                          numArchs = getNumArchs(header);
                          offset = FAT_HEADER_BYTES;
                          archNum = 0;

                        case 14:
                          if (!(archNum < numArchs)) {
                            _context2.next = 32;
                            break;
                          }

                          _context2.next = 17;
                          return readBytes(fd, offset, FAT_ARCH_BYTES, 'Error reading details for arch ' + archNum + ' in multi-arch mapping');

                        case 17:
                          fatArchHeader = _context2.sent;
                          _getArchDetails = getArchDetails(fatArchHeader, archNum), cpuType = _getArchDetails.cpuType, cpuSubType = _getArchDetails.cpuSubType, archOffset = _getArchDetails.archOffset;
                          archName = getArchName(cpuType, cpuSubType);


                          offset += FAT_ARCH_BYTES;

                          _context2.next = 23;
                          return readBytes(fd, archOffset, MAGIC_NUMBER_BYTES);

                        case 23:
                          magicDataView = _context2.sent;

                          magic = parseMagicNumber(magicDataView, archNum, archName);

                          _context2.next = 27;
                          return getArchEntry(fd, magic, archOffset, archNum, archName);

                        case 27:
                          archEntry = _context2.sent;

                          archEntries.push(archEntry);

                        case 29:
                          archNum++;
                          _context2.next = 14;
                          break;

                        case 32:
                          resolve(archEntries);
                          _context2.next = 39;
                          break;

                        case 35:
                          _context2.next = 37;
                          return getArchEntry(fd, magic);

                        case 37:
                          _archEntry = _context2.sent;

                          resolve([_archEntry]);

                        case 39:
                          _context2.next = 44;
                          break;

                        case 41:
                          _context2.prev = 41;
                          _context2.t0 = _context2['catch'](0);

                          reject(_context2.t0);

                        case 44:
                        case 'end':
                          return _context2.stop();
                      }
                    }
                  }, _callee2, _this, [[0, 41]]);
                }));

                return function (_x12, _x13) {
                  return _ref3.apply(this, arguments);
                };
              }());
            }));

          case 1:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));

  return function getMachOArchs(_x11) {
    return _ref2.apply(this, arguments);
  };
}();

var _fs = require('fs');

var _errorTypes = require('./errorTypes.js');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var MAGIC_NUMBERS = {
  MH_MAGIC_64: 0xfeedfacf,
  MH_CIGAM_64: 0xcffaedfe,
  MH_MAGIC: 0xfeedface,
  MH_CIGAM: 0xcefaedfe,
  FAT_MAGIC: 0xcafebabe,
  FAT_CIGAM: 0xbebafeca
};

var MAGIC_NUMBER_VALS = (0, _values2.default)(MAGIC_NUMBERS);

var CPU_TYPES = {
  I386: 0x00000007,
  X86_64: 0x01000007,
  ARM: 0x0000000c,
  ARM64: 0x0100000c
};
var ARCH_BASE_NAMES = (_ARCH_BASE_NAMES = {}, (0, _defineProperty3.default)(_ARCH_BASE_NAMES, CPU_TYPES.I386, 'i386'), (0, _defineProperty3.default)(_ARCH_BASE_NAMES, CPU_TYPES.X86_64, 'x86_64'), (0, _defineProperty3.default)(_ARCH_BASE_NAMES, CPU_TYPES.ARM, 'arm'), (0, _defineProperty3.default)(_ARCH_BASE_NAMES, CPU_TYPES.ARM64, 'arm64'), _ARCH_BASE_NAMES);

var CPU_SUB_TYPES = {
  ALL_LE: 0x00000003,
  ALL_BE: 0x00000000,
  H: 0x00000008,
  V5: 0x00000007,
  V6: 0x00000006,
  V6M: 0x0000000e,
  V7: 0x00000009,
  V7F: 0x0000000a,
  V7S: 0x0000000b,
  V7K: 0x0000000c,
  V7M: 0x0000000f,
  V7EM: 0x00000010,
  V8: 0x0000000d,
  E: 0x00000002
};
var ARCH_SUB_NAMES = (_ARCH_SUB_NAMES = {}, (0, _defineProperty3.default)(_ARCH_SUB_NAMES, CPU_SUB_TYPES.ALL_LE, ''), (0, _defineProperty3.default)(_ARCH_SUB_NAMES, CPU_SUB_TYPES.ALL_BE, ''), (0, _defineProperty3.default)(_ARCH_SUB_NAMES, CPU_SUB_TYPES.H, 'h'), (0, _defineProperty3.default)(_ARCH_SUB_NAMES, CPU_SUB_TYPES.V5, 'v5'), (0, _defineProperty3.default)(_ARCH_SUB_NAMES, CPU_SUB_TYPES.V6, 'v6'), (0, _defineProperty3.default)(_ARCH_SUB_NAMES, CPU_SUB_TYPES.V6M, 'v6m'), (0, _defineProperty3.default)(_ARCH_SUB_NAMES, CPU_SUB_TYPES.V7, 'v7'), (0, _defineProperty3.default)(_ARCH_SUB_NAMES, CPU_SUB_TYPES.V7F, 'v7f'), (0, _defineProperty3.default)(_ARCH_SUB_NAMES, CPU_SUB_TYPES.V7S, 'v7s'), (0, _defineProperty3.default)(_ARCH_SUB_NAMES, CPU_SUB_TYPES.V7K, 'v7k'), (0, _defineProperty3.default)(_ARCH_SUB_NAMES, CPU_SUB_TYPES.V7M, 'v7m'), (0, _defineProperty3.default)(_ARCH_SUB_NAMES, CPU_SUB_TYPES.V7EM, 'v7em'), (0, _defineProperty3.default)(_ARCH_SUB_NAMES, CPU_SUB_TYPES.V8, 'v8'), (0, _defineProperty3.default)(_ARCH_SUB_NAMES, CPU_SUB_TYPES.E, 'e'), _ARCH_SUB_NAMES);

var UNKNOWN_ARCH_TYPE = 'unknown';

var LC_UUID = 0x1b;
var UUID_BYTES = 16;

var MAGIC_NUMBER_BYTES = 4;
var MACH_HEADER_BYTES = 28;
var MACH_64_HEADER_BYTES = 32;

var FAT_ARCH_BYTES = 20;
var FAT_HEADER_BYTES = 8;

var LOAD_COMMAND_BYTES = 8;

function readBytes(fd, position, length, errMessage) {
  var asDataView = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : true;

  return new _promise2.default(function (resolve) {
    var output = Buffer.alloc(length);
    (0, _fs.read)(fd, output, 0, length, position, function (err) {
      if (err) {
        (0, _errorTypes.handleFileError)(errMessage, err);
      }
      if (asDataView) {
        var dataView = new DataView(output.buffer);
        resolve(dataView);
      } else {
        resolve(output);
      }
    });
  });
}

function getUint32(buffer, offset, errMessage) {
  var littleEndian = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

  try {
    return buffer.getUint32(offset, littleEndian);
  } catch (err) {
    if (err instanceof RangeError) {
      throw new _errorTypes.BufferRangeError(errMessage, err);
    }
    throw new _errorTypes.DataViewError(errMessage, err);
  }
}

function getErrorSuffixForArch(archNum) {
  var archName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

  if (archNum !== null) {
    return 'for arch ' + archNum + (archName ? ' (' + archName + ')' : '') + ' in multi-arch mapping';
  }
  return 'for ' + (archName ? archName + ' ' : '') + 'arch in single-arch mapping';
}

function parseMagicNumber(view) {
  var archNum = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
  var archName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

  var suffix = archName === null ? 'for file' : getErrorSuffixForArch(archNum, archName);
  var magic = getUint32(view, 0, 'Error parsing magic number ' + suffix);
  if (!MAGIC_NUMBER_VALS.includes(magic)) {
    throw new _errorTypes.MagicNumberError('Invalid magic number: ' + magic + ' ' + suffix);
  }
  return magic;
}

function getIsMach64Header(magic) {
  return magic === MAGIC_NUMBERS.MH_MAGIC_64 || magic === MAGIC_NUMBERS.MH_CIGAM_64;
}

function shouldSwapBytes(magic) {
  return magic === MAGIC_NUMBERS.MH_CIGAM || magic === MAGIC_NUMBERS.MH_CIGAM_64;
}

function getIsMultiArch(magic) {
  return magic === MAGIC_NUMBERS.FAT_CIGAM || magic === MAGIC_NUMBERS.FAT_MAGIC;
}

function getNumArchs(view) {
  return getUint32(view, 4, 'Error parsing arch count for multi-arch mapping');
}

function getArchDetails(view, archNum) {
  var archSuffix = getErrorSuffixForArch(archNum);

  var cpuType = getUint32(view, 0, 'Error parsing cpuType ' + archSuffix);
  var cpuSubType = getUint32(view, 4, 'Error parsing cpuSubType ' + archSuffix);
  var archOffset = getUint32(view, 8, 'Error parsing offset ' + archSuffix);

  return { cpuType: cpuType, cpuSubType: cpuSubType, archOffset: archOffset };
}

function getHeaderVals(view, shouldSwap, archSuffix) {
  var cpuType = getUint32(view, 4, 'Error parsing cpuType ' + archSuffix, shouldSwap);
  var cpuSubType = getUint32(view, 8, 'Error parsing cpuSubType ' + archSuffix, shouldSwap);

  var ncmds = getUint32(view, 16, 'Error parsing number of load commands ' + archSuffix, shouldSwap);

  return { cpuType: cpuType, cpuSubType: cpuSubType, ncmds: ncmds };
}

function getArchName(cpuType, cpuSubType) {
  var base = ARCH_BASE_NAMES[cpuType];
  if (base) {
    var subArch = ARCH_SUB_NAMES[cpuSubType] || '';
    return '' + base + subArch;
  }
  return UNKNOWN_ARCH_TYPE;
}

function getLoadCommandDetails(view, shouldSwap, cmdSuffix) {
  var cmd = getUint32(view, 0, 'Error parsing ' + cmdSuffix, shouldSwap);
  var cmdSize = getUint32(view, 4, 'Error parsing size of ' + cmdSuffix, shouldSwap);

  return { cmd: cmd, cmdSize: cmdSize };
}