'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.uploadMachO = exports.missingUUIDError = exports.magicNumberError = undefined;

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _fs = require('fs');

var _apiClient = require('./apiClient');

var _formatError = require('./formatError.js');

var _gatherFiles = require('./gatherFiles.js');

var _sortMachOFiles = require('./sortMachOFiles.js');

var _errorTypes = require('./errorTypes.js');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var dsymDocsLink = 'https://developer.apple.com/documentation/xcode/building-your-app-to-include-debugging-information/';
var fileCountError = 'No debug information files found\nSee ios dev docs for information on building your app to include debug info ' + dsymDocsLink + '\n';

var gatherFilesError = 'An error occurred while gathering debug files';

var outOfRangeError = 'An out of range error occurred while trying to read bytes from mapping file';
var fileAccessError = 'Incorrect permissions for path';
var fileNotFoundError = 'No such file or directory';
var timedOutError = 'Operation timed out while parsing mapping file';
var readError = 'An error occurred while trying to read from mapping file';
var bufferRangeError = 'Attempted to access bytes outside of range while parsing mapping file';
var dataViewError = 'An error occurred while parsing bytes from mapping file';
var magicNumberError = exports.magicNumberError = 'File may not be of expected type (MachO), found an invalid magic number for';
var missingUUIDError = exports.missingUUIDError = 'No uuid found for mapping file';
var loadCommandError = 'File may not be of expected type (MachO), an error occurred reading load commands from';
var sortingError = 'An error occurred while parsing architecture details from mapping file';

var getFileErrMessage = function getFileErrMessage(err) {
  switch (err.name) {
    case _errorTypes.ERROR_NAMES.AccessError:
      return fileAccessError;
    case _errorTypes.ERROR_NAMES.FileNotFoundError:
      return fileNotFoundError;
    case _errorTypes.ERROR_NAMES.TimedOutError:
      return timedOutError;
    case _errorTypes.ERROR_NAMES.ReadFileError:
      return readError;
    default:
      return null;
  }
};

var uploadMachO = exports.uploadMachO = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(args) {
    var paths, release, apikey, apihost, verbose, client, uploadFile, fileList, errDetail, errMessage, archEntriesLists, archEntries, CHUNK_SIZE, i;
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            paths = args.paths, release = args.release, apikey = args.apikey, apihost = args.apihost, verbose = args.verbose;


            console.info('Preparing to upload debug file(s) for release ' + release + ' ...');
            console.info('Gathering file list...');

            client = (0, _apiClient.apiClient)({ apikey: apikey, apihost: apihost });
            _context3.next = 6;
            return client.checkStatus();

          case 6:

            if (args['gcs-token']) {
              client.setGCSData({
                gcsToken: args['gcs-token'],
                gcsBucket: args['gcs-bucket']
              });
            }

            uploadFile = function () {
              var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(_ref3) {
                var arch = _ref3.arch,
                    fileFormat = _ref3.fileFormat,
                    name = _ref3.name,
                    path = _ref3.path,
                    uuid = _ref3.uuid;
                var containerDirectory, debugFilePath, debugFileData, metaFilePath, meta, metaFileData, debugFileRes, metaFileRes;
                return _regenerator2.default.wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        console.info('Uploading: ' + name + ' ' + arch);

                        containerDirectory = uuid.slice(0, 2) + '/' + uuid.slice(2);
                        debugFilePath = containerDirectory + '/debuginfo';
                        debugFileData = {
                          contents: (0, _fs.createReadStream)(path),
                          data: { filepath: debugFilePath, release: release },
                          maxRetries: args['max-retries'],
                          maxRetryDelay: args['max-retry-delay'],
                          url: 'release-artifacts'
                        };
                        metaFilePath = containerDirectory + '/meta';
                        meta = {
                          name: name,
                          arch: arch,
                          file_format: fileFormat
                        };
                        metaFileData = {
                          contents: (0, _stringify2.default)(meta),
                          data: { filepath: metaFilePath, release: release },
                          maxRetries: args['max-retries'],
                          maxRetryDelay: args['max-retry-delay'],
                          url: 'release-artifacts'
                        };
                        _context.prev = 7;
                        _context.next = 10;
                        return client.uploadFile(debugFileData);

                      case 10:
                        debugFileRes = _context.sent;

                        if (debugFileRes.ok) {
                          _context.next = 15;
                          break;
                        }

                        console.error('Failed to upload: ' + name + ' ' + arch);
                        _context.next = 15;
                        return (0, _formatError.formatError)(debugFileRes, { verbose: verbose });

                      case 15:
                        _context.next = 17;
                        return client.uploadFile(metaFileData);

                      case 17:
                        metaFileRes = _context.sent;

                        if (metaFileRes.ok) {
                          _context.next = 22;
                          break;
                        }

                        console.error('Failed to upload metadata for ' + name + ' ' + arch);
                        _context.next = 22;
                        return (0, _formatError.formatError)(metaFileRes, { verbose: verbose });

                      case 22:
                        _context.next = 28;
                        break;

                      case 24:
                        _context.prev = 24;
                        _context.t0 = _context['catch'](7);

                        console.error(_context.t0.message);
                        process.exit(1);

                      case 28:
                      case 'end':
                        return _context.stop();
                    }
                  }
                }, _callee, undefined, [[7, 24]]);
              }));

              return function uploadFile(_x2) {
                return _ref2.apply(this, arguments);
              };
            }();

            fileList = void 0;
            _context3.prev = 9;
            _context3.next = 12;
            return (0, _gatherFiles.gatherFiles)(paths, { globString: '**/DWARF/*' });

          case 12:
            fileList = _context3.sent;
            _context3.next = 21;
            break;

          case 15:
            _context3.prev = 15;
            _context3.t0 = _context3['catch'](9);
            errDetail = getFileErrMessage(_context3.t0);
            errMessage = '' + gatherFilesError + (errDetail ? ': ' + errDetail : '');


            if (verbose) {
              console.error(errMessage + '\n' + _context3.t0.stack);
            } else {
              console.error(errMessage + '\nFor additional details, rerun command with --verbose');
            }
            process.exit(1);

          case 21:

            if (fileList.length === 0) {
              console.error(fileCountError);
              process.exit(1);
            }

            console.info('Found ' + fileList.length + ' debug file' + (fileList.length === 1 ? '' : 's') + ' ...');

            if (verbose) {
              console.info(fileList.map(function (_ref4) {
                var path = _ref4.path;
                return '- ' + path;
              }).join('\n'));
            } else {
              console.info('Rerun command with --verbose to see debug file paths');
            }

            _context3.next = 26;
            return _promise2.default.all(fileList.map(function () {
              var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(_ref6) {
                var name = _ref6.name,
                    path = _ref6.path;

                var fileArchEntries, _errMessage;

                return _regenerator2.default.wrap(function _callee2$(_context2) {
                  while (1) {
                    switch (_context2.prev = _context2.next) {
                      case 0:
                        fileArchEntries = void 0;
                        _context2.prev = 1;
                        _context2.next = 4;
                        return (0, _sortMachOFiles.getMachOArchs)(path);

                      case 4:
                        fileArchEntries = _context2.sent;
                        _context2.next = 30;
                        break;

                      case 7:
                        _context2.prev = 7;
                        _context2.t0 = _context2['catch'](1);
                        _errMessage = getFileErrMessage(_context2.t0);

                        if (!(_errMessage === null)) {
                          _context2.next = 26;
                          break;
                        }

                        _context2.t1 = _context2.t0.name;
                        _context2.next = _context2.t1 === _errorTypes.ERROR_NAMES.OutOfRangeError ? 14 : _context2.t1 === _errorTypes.ERROR_NAMES.BufferRangeError ? 15 : _context2.t1 === _errorTypes.ERROR_NAMES.DataViewError ? 17 : _context2.t1 === _errorTypes.ERROR_NAMES.MagicNumberError ? 19 : _context2.t1 === _errorTypes.ERROR_NAMES.LoadCommandError ? 21 : _context2.t1 === _errorTypes.ERROR_NAMES.MissingUUIDError ? 23 : 25;
                        break;

                      case 14:
                        return _context2.abrupt('return', outOfRangeError);

                      case 15:
                        _errMessage = bufferRangeError;
                        return _context2.abrupt('break', 26);

                      case 17:
                        _errMessage = dataViewError;
                        return _context2.abrupt('break', 26);

                      case 19:
                        _errMessage = magicNumberError;
                        return _context2.abrupt('break', 26);

                      case 21:
                        _errMessage = loadCommandError;
                        return _context2.abrupt('break', 26);

                      case 23:
                        _errMessage = missingUUIDError;
                        return _context2.abrupt('break', 26);

                      case 25:
                        _errMessage = sortingError;

                      case 26:
                        _errMessage += ' ' + path;

                        if (verbose) {
                          _errMessage += '\n' + _context2.t0.stack;
                        } else {
                          _errMessage += '\nFor additional details, rerun command with --verbose';
                        }
                        console.error(_errMessage);
                        process.exit(1);

                      case 30:
                        return _context2.abrupt('return', fileArchEntries.map(function (entry) {
                          return (0, _extends3.default)({}, entry, { name: name, path: path });
                        }));

                      case 31:
                      case 'end':
                        return _context2.stop();
                    }
                  }
                }, _callee2, undefined, [[1, 7]]);
              }));

              return function (_x3) {
                return _ref5.apply(this, arguments);
              };
            }()));

          case 26:
            archEntriesLists = _context3.sent;
            archEntries = [];

            archEntriesLists.forEach(function (list) {
              return archEntries.push.apply(archEntries, (0, _toConsumableArray3.default)(list));
            });
            console.info('Parsed ' + archEntries.length + ' total build architecture mappings');

            CHUNK_SIZE = 1;
            i = 0;

          case 32:
            if (!(i < archEntries.length)) {
              _context3.next = 38;
              break;
            }

            _context3.next = 35;
            return _promise2.default.all(archEntries.slice(i, i + CHUNK_SIZE).map(uploadFile));

          case 35:
            i += CHUNK_SIZE;
            _context3.next = 32;
            break;

          case 38:

            console.info('Success!');

          case 39:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, undefined, [[9, 15]]);
  }));

  return function uploadMachO(_x) {
    return _ref.apply(this, arguments);
  };
}();