'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.uploadProguard = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _fs = require('fs');

var _apiClient = require('./apiClient.js');

var _formatError = require('./formatError.js');

var _path = require('path');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var sampleAndroidUploadPath = '/app/build/outputs/mapping/debug/mapping.txt';
var pathCountError = 'Include one path directly to debug file, like `' + sampleAndroidUploadPath + '`\nSee android dev docs for information on how to shrink and obfuscate your code https://developer.android.com/build/shrink-code#enable';

var uploadProguard = exports.uploadProguard = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(args) {
    var paths, release, apikey, apihost, verbose, path, client, fileStats, data, res;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            paths = args.paths, release = args.release, apikey = args.apikey, apihost = args.apihost, verbose = args.verbose;

            if (paths.length > 1) {
              console.error('More than 1 proguard file found, ' + pathCountError);
              process.exit(1);
            }
            path = paths[0];


            console.info('Preparing to upload ProGuard mapping for release ' + release + ' ...');

            client = (0, _apiClient.apiClient)({ apikey: apikey, apihost: apihost });
            _context.next = 7;
            return client.checkStatus();

          case 7:

            if (args['gcs-token']) {
              client.setGCSData({
                gcsToken: args['gcs-token'],
                gcsBucket: args['gcs-bucket']
              });
            }

            fileStats = (0, _fs.statSync)(path, { throwIfNoEntry: false });

            if (!fileStats) {
              console.error(path + ' does not exist');
              process.exit(1);
            }

            if (!fileStats.isFile()) {
              console.error(path + ' is not a file, ' + pathCountError);
              process.exit(1);
            }

            if ((0, _path.extname)(path) !== '.txt') {
              console.error('ProGuard mapping file must be .txt');
              process.exit(1);
            }

            data = {
              contents: (0, _fs.createReadStream)(path),
              data: { filepath: 'mapping.txt', release: release },
              maxRetries: args['max-retries'],
              maxRetryDelay: args['max-retry-delay'],
              url: 'release-artifacts'
            };
            _context.prev = 13;
            _context.next = 16;
            return client.uploadFile(data);

          case 16:
            res = _context.sent;

            if (res.ok) {
              _context.next = 23;
              break;
            }

            console.error('Failed to upload mapping.txt');
            _context.next = 21;
            return (0, _formatError.formatError)(res, { verbose: verbose });

          case 21:
            _context.next = 24;
            break;

          case 23:
            console.info('Success!');

          case 24:
            _context.next = 30;
            break;

          case 26:
            _context.prev = 26;
            _context.t0 = _context['catch'](13);

            console.error(_context.t0.message);
            process.exit(1);

          case 30:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined, [[13, 26]]);
  }));

  return function uploadProguard(_x) {
    return _ref.apply(this, arguments);
  };
}();